import getpass
import urllib
import urllib2
import os
from time import sleep


def user_input():
    base_url = "".join(
        raw_input('Enter the url where you access alation, including http:// (e.g. http://alation.mycorp.com/)\n URL: ').split())
    base_url = base_url.strip("/")
    username = "".join(raw_input('Enter your email:  ').split())
    password = getpass.getpass('Enter your password:  ')
    return (base_url, username, password)


def token_request(request_type, values=()):
    if request_type == "getToken":
        (base_url, username, password) = user_input()
        values = (base_url, username, password)
    else:
        (base_url, username, password) = values
    request = urllib2.Request(
        base_url + "/api/" + request_type + "/")
    try:
        data = urllib.urlencode({"Username": username, "Password": password})
        response = urllib2.urlopen(request, data=data)
        reply = response.read()
        if len(reply) > 36:
            # execute_request will catch unknown error
            print response.status, response.reason
        return (reply, values)
    except urllib2.URLError:
        return ("NOT_OPEN", ())
    except ValueError:
        return ("BAD_FORMAT", ())


def retry(message, values):
    yorn = raw_input(
        message + "\n(y/[n]): ")
    if yorn == "y":
        execute_request(token_request("getToken", values))


def execute_request(reply):
    (status, values) = reply
    if status == "EXISTING":
        yorn = raw_input(
            "Key already exists on server. " +
            "Would you like to change your key and configure the api on this machine? \n(y/[n]): ")
        if yorn == "y":
            execute_request(token_request("changeToken", values))
    elif status == "NOT_EXISTING":
        retry("Key does not exist. Would you like to try to create one?", values)
    elif status == "INVALID":
        retry("Invalid username or password. Would you like to try again?", values)
    elif status == "NOT_OPEN":
        retry("Could not open URL. Make sure it starts with http:// Would you like to try again?", values)
    elif status == "BAD_FORMAT":
        retry("Incorrectly formatted URL. Make sure it starts with http:// Would you like to try again?", values)
    elif len(status) == 36:
        print "Creating config file with API key " + status + " ..."
        generate_config_file(status + "\n" + values[0])
        print "Success!"
    else:
        print "Error. Please check if the API url you entered is correct and starts with http://"
        retry("Would you like to try again?", values)


def generate_config_file(token):
    home = os.path.expanduser('~')
    doc = open(home + "/.alation_api_config", "w")
    doc.write(token)
    doc.close()


def run_setup():
    print "This quick script will set up your API key in a config file"
    sleep(1.0)
    execute_request(token_request("getToken"))



import urllib2
import os
import re
import StringIO


class AlationError(Exception):
    pass


def handle_exception(code):
    if code == 404:
        raise AlationError(
            "404: Your base url may be wrong, or the query with that id may not exist")
    elif code == 403:
        raise AlationError(
            "403: Permission denied. If this is incorrect, try regenerating your API token")
    else:
        raise AlationError("HTTP Error: " + str(code))


def read_config_file():
    try:
        home = os.path.expanduser('~')
        f = open(home + "/.alation_api_config", "r")
        token = f.readline().rstrip()
        base_url = f.readline().rstrip()
        return (token, base_url)
    except IOError:
        raise AlationError(
            "Can't find your alation_api_config file. Please run alation-setup")


def make_request(request_id, request_type, response_type):
    (token, base_url) = read_config_file()
    request_url = "/api/" + request_type + \
        "/" + str(request_id) + "/" + response_type + "/"
    request = urllib2.Request(base_url + request_url)
    request.add_header("token", token)
    return request


def get_sql(qid):
    """Returns SQL of a query based on its unique query id.
    """
    request = make_request(qid, "query", "sql")
    try:
        response = urllib2.urlopen(request)
        # probably won't actually be used, but just in case
        code = response.getcode()
        if code != 200:
            handle_exception(code)
        return response.read()
    except urllib2.HTTPError, error:
        handle_exception(error.code)


def get_result(rid):
    """Returns CSV of a particular ExecutionResult from its unique id.
    """
    request = make_request(rid, "result", "csv")
    try:
        response = urllib2.urlopen(request)
        output = StringIO.StringIO()
        CHUNK = 16 * 1024
        while True:
            chunk = response.read(CHUNK)
            if not chunk:
                break
            output.write(chunk)
        return output.getvalue()
    except urllib2.HTTPError, error:
        handle_exception(error.code)


def subbed_query(qid, replacements):
    sql = get_sql(qid)
    for key in replacements:
        sql = re.sub(u"\$\{" + key + "\}", replacements[key], sql)
    return sql



import api_setup_client as api_setup
import api_request_client as api
from api_request_client import AlationError
import sys


def print_sql(*args):
    try:
        qid = int(sys.argv[1])
        print api.get_sql(qid)
    except ValueError:
        print "Not a valid integer for query id"
    except AlationError, error:
        print error


def print_result(*args):
    try:
        rid = int(sys.argv[1])
        print api.get_result(rid)
    except ValueError:
        print "Not a valid integer for result id"
    except AlationError, error:
        print error


def setup():
    api_setup.run_setup()



from setuptools import setup


setup(
    name='alation',
    version='0.1.12',
    packages=['alation'],
    entry_points={
        'console_scripts': [
            'alation-setup = alation.main:setup',
            'alation-query = alation.main:print_sql',
            'alation-result = alation.main:print_result'
        ]
    }
)
